package org.steveshipway.blueprints;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
//import net.minecraft.util.BlockPos;

import org.steveshipway.blueprints.util17.*;

import org.apache.logging.log4j.Logger;
import org.steveshipway.blueprints.directive.*;

public class Ruins {

	// Import template in TML format, and install into Portfolio 
	public static boolean importTmlFile(Portfolio portfolio,File fileEntry) {
		Blueprint bp;
		String name = "ruin";
		String biomes = "All";
		String description = "Imported from Ruins template";
		LinkedList steps = new LinkedList();
	    BufferedReader br;
	    int lineNo = 0;
		Writer writer = null;
		String cvtFileName;
		int width, height, length;
    	Pattern pSet  = Pattern.compile("^([^=]+)=(\\S+)",Pattern.CASE_INSENSITIVE);
    	Pattern pRule = Pattern.compile("^rule(\\d+)=(\\d),(\\d+),(\\S+)",Pattern.CASE_INSENSITIVE);
    	Pattern pBlock = Pattern.compile("([^\\s-]+)(-(\\d+))?(-.*)?");
    	Matcher m;
    	Directive d;
    	int auto_chance = 0;
    	int i,j;
    	int layer = 0;
    	int embed = 0;
    	int maxlevel = 0;
    	ArrayList<Integer> ruleOption = new ArrayList<Integer>(); 

    	ruleOption.add(0,0); // air block render first
    	
    	if( ! portfolio.quiet ) { portfolio.logger.info("["+fileEntry.getName()+"]: Parsing Ruins template file "); }

		// Create conversion file
		name = fileEntry.getName().replace(".tml","");
		cvtFileName = fileEntry.getPath().replace(".tml",".bp");

		// open tml file
		try {
			br = new BufferedReader(new FileReader(fileEntry));
		} catch (FileNotFoundException e) {
			portfolio.logger.error("["+fileEntry.getName()+"]: Unable to open file: "+e.getMessage());
			return false;
		}
		// open conversion file
		try {
		    writer = new BufferedWriter(new OutputStreamWriter(
		          new FileOutputStream(cvtFileName), "utf-8")
		    );
		    writer.write("# Blueprint for: "+ name +"\n");
		    writer.write("# Converted from Ruins template\n");
		    writer.write("\n");
		} catch (IOException ex) {
			portfolio.logger.error("Unable to write file "+cvtFileName+": "+ex.getMessage());
			try {writer.close();} catch(Exception ex2) {}
			writer = null;
		}
		// process file
		try {
	        String line;
	        layer = 0;
	        height = length = width = 0;
	        while ((line = br.readLine()) != null) {
	        	// work through lines in the file
	        	line = line.trim();
	        	if( line.isEmpty() || line.startsWith("#") ) { continue; }
	        	// possible lines:
	        	// rule#=option,chance,blockname[-dv][,blockname[-dv]]...
	        	if( line.startsWith("rule")) {
	        		m = pRule.matcher(line);
	        		if( m.find() ) {
	        			List directives = new LinkedList();
		        		d = new NullDirective();
	        			// define a lookup item
	        			int chance = Integer.parseInt(m.group(3));
	        			int ruleNo = Integer.parseInt(m.group(1));
	        			// group(2) controls tests for nearby blocks, and sequence
	        			ruleOption.add(ruleNo,Integer.parseInt(m.group(2))); 
	        			// group(4) is the block outcome list
	        			for( String b : m.group(4).split(",")) {
        					if( b.matches("preserveBlock") ) {
	        					d = new NullDirective();
	        				} else if( b.startsWith("CommandBlock")) { // CommandBlock:command:sender
	        					Pattern cmdblock = Pattern.compile("CommandBlock:(.*):(\\S+)",Pattern.CASE_INSENSITIVE);
	        					m = cmdblock.matcher(b);
	        					if(m.find() ) {
	        						d = new BlockDirective(Blocks.command_block,0,false,new BlockPos(0,0,0),"{id=\"Control\",Command=\""+m.group(1)+"\",}");
	        					} else {
	        						d = new BlockDirective(Blocks.command_block,0,false,new BlockPos(0,0,0),"{id=\"Control\",}");
	        					}
	        				} else if( b.startsWith("StandingSign")) { // StandingSign:a|b|c|d-4
	        					Pattern ssign = Pattern.compile("StandingSign:(.*)\\|(.*)\\|(.*)\\|(.*)(-(\\d+))?(-.*)?",Pattern.CASE_INSENSITIVE);	        	
	        					m = ssign.matcher(b);
	        					if(m.find() ) {
	        						int dv = Integer.parseInt(m.group(6));
	        						String text1 = "";
	        						String text2 = "";
	        						String text3 = "";
	        						String text4 = "";
	        						if( m.group(1) != null && !m.group(1).matches("null")) { text1 = m.group(1); }
	        						if( m.group(2) != null && !m.group(2).matches("null")) { text2 = m.group(2); }
	        						if( m.group(3) != null && !m.group(3).matches("null")) { text3 = m.group(3); }
	        						if( m.group(4) != null && !m.group(4).matches("null")) { text4 = m.group(4); }
	        						d = new BlockDirective(Blocks.standing_sign,dv,false,new BlockPos(0,0,0),
	        								"{id=\"Sign\",Text1=\""+text1+"\",Text2=\""+text2+"\",Text3=\""+text3+"\",Text4=\""+text4+"\",}");
	        					} else {
	        						d = new BlockDirective(Blocks.standing_sign,0,false,new BlockPos(0,0,0),
	        								"{id=\"Sign\",Text1=\"Parse\",Text2=\"Error\",Text3=\"in\",Text4=\"template\",}");
	        					}
	        				} else if( b.startsWith("IInventory")) {  // IInventory;chest;red_flower+arrow#10+wool#3#15-5
	        					Pattern inv = Pattern.compile("IInventory;([^;]+);([^-]*)(-(\\d+))?(-.*)?",Pattern.CASE_INSENSITIVE);	        	
	        					m = inv.matcher(b);
	        					if(m.find() ) {
	        						int dv = Integer.parseInt(m.group(4));
	        						Block blk = Block.getBlockFromName(m.group(1));
	        						if( blk == null ) { blk = Block.getBlockFromName("minecraft:"+m.group(1)); }
	        						if( blk != null ) {
	        							String nbt = inventoryToNBT(blk,m.group(2), portfolio.logger);
	        							d = new BlockDirective(blk,dv,false,new BlockPos(0,0,0),nbt);
	        						} else {
		        						portfolio.logger.warn("Invalid block name: "+m.group(1));	        					
	        							d = new BlockDirective(Blocks.air,0,false,new BlockPos(0,0,0),null);
	        						}
	        					} else {
	        						portfolio.logger.warn("Unable to parse IInventory tag: "+b);
	        						d = new NullDirective();
	        					}
	        				} else if( b.startsWith("HardChest") || b.startsWith("MediumChest") || b.startsWith("EasyChest")) { // ChestGenHook:dungeonChest:10 
        						d = new ChestDirective("CHEST 0,0,0  0  ChestGenHooks:dungeonChest,10",portfolio.path,portfolio.logger);
	        					Pattern chestgen = Pattern.compile("(\\S+Chest)(-(\\d))?(-.*)?",Pattern.CASE_INSENSITIVE);
	        					m = chestgen.matcher(b);
	        					if(m.find() ) {
	        						int rot = 0;
	        						try { rot = Integer.parseInt(m.group(4)); } catch( Exception e ) {}
	        						d = new ChestDirective("CHEST 0,0,0 "+rot+" "+m.group(1)+".nbt",portfolio.path,portfolio.logger);
	        					} else {
	        						d = new ChestDirective("CHEST 0,0,0 0  ChestGenHooks:dungeonChest,10",portfolio.path,portfolio.logger);	        						
	        					}
	        				} else if( b.startsWith("ChestGenHook") ) { // ChestGenHook:dungeonChest:10 
	        					Pattern chestgen = Pattern.compile("ChestGenHook:(\\S+):(\\d+)(-(\\d))?(-.*)?",Pattern.CASE_INSENSITIVE);
	        					m = chestgen.matcher(b);
	        					if(m.find() ) {
	        						int rot = 0;
	        						try { rot = Integer.parseInt(m.group(4)); } catch( Exception e ) {}
	        						d = new ChestDirective("CHEST 0,0,0 "+rot+" ChestGenHooks:"+m.group(1)+","+m.group(2),portfolio.path,portfolio.logger);
	        					} else {
	        						d = new ChestDirective("CHEST 0,0,0 0",portfolio.path,portfolio.logger);	        						
	        					}
	        				} else if( b.startsWith("Skull")) { // Skull:0:0[:uuid-name][-0] 
	        					Pattern skull = Pattern.compile("Skull:(\\d+):(\\d+)(:(\\S+)-(\\S+))?(-(\\d))?",Pattern.CASE_INSENSITIVE);
	        					m = skull.matcher(b);
        						int dv = 0;
	        					if(m.find() ) {
	        						if( m.group(7) != null ) {
	        							try { dv = Integer.parseInt(m.group(7)); } catch( Exception e ) {}
	        						}
	        						d = new BlockDirective(Blocks.skull,dv,false,new BlockPos(0,0,0),"{id=\"Skull\",SkullType="+m.group(1)+"b,Rot="+m.group(2)+"b,"
	        								+"Owner={Id=\""+m.group(4)+"\",Name=\""+m.group(5)+"\",},}");
	        					} else {
	        						d = new BlockDirective(Blocks.skull,dv,false,new BlockPos(0,0,0),"{id=\"Skull\",SkullType=0b,Rot=0b,}");
	        					}
	        				} else if( b.startsWith("EasyMobSpawn") ||b.startsWith("MediumMobSpawn") ||b.startsWith("HardMobSpawn") || b.startsWith("UprightMobSpawn")	) {
	        					Pattern spawner = Pattern.compile("(\\S+Spawn)(-.*)?",Pattern.CASE_INSENSITIVE);
	        					m = spawner.matcher(b);
	        					if(m.find() ) {
	        						d = new SpawnerDirective("SPAWNER 0 0 0 "+m.group(1)+".nbt",portfolio.path,portfolio.logger);
	        					} else {
	        						d = new SpawnerDirective("SPAWNER 0 0 0 Zombie",portfolio.path,portfolio.logger);	        						
	        					}
	        				} else if( b.startsWith("MobSpawner") ) { // MobSpawner:mobname
	        					Pattern spawner = Pattern.compile("MobSpawner:([^-\\s]*)(-.*)?",Pattern.CASE_INSENSITIVE);
	        					m = spawner.matcher(b);
	        					if(m.find() ) {
	        						d = new SpawnerDirective("SPAWNER 0 0 0 "+m.group(1),portfolio.path,portfolio.logger);
	        					} else {
	        						portfolio.logger.warn("Invalid MobSPawner potential in Ruins template: "+b);
	        						d = new SpawnerDirective("SPAWNER 0 0 0 Chicken",portfolio.path,portfolio.logger);	        						
	        					}
	        				} else {
	        					m = pBlock.matcher(b);
	        					if( m.find() ) {
	        						int dv = 0;
	        						if( m.group(3) != null ) {
	        							try { dv = Integer.parseInt(m.group(3)); } catch( Exception e) {}
	        						}
	        						Block block;
	        						block = Block.getBlockFromName(m.group(1));
	        						if( block == null ) {
		        						block = Block.getBlockFromName("minecraft:"+m.group(1));	        							
	        						}
	        						if( block != null ) {
	        							d = new BlockDirective(block,dv,false,new BlockPos(0,0,0),null);
	        						} else {
	        							portfolio.logger.warn("Unrecognised block name: "+m.group(1));
	        							d = new BlockDirective(Blocks.air,0,false,new BlockPos(0,0,0),null);	        							
	        						}
	        					} else {
        							portfolio.logger.warn("Unrecognised block name: "+b);
        							d = new BlockDirective(Blocks.air,0,false,new BlockPos(0,0,0),null);	        							
	        					}
	        				}
	        				
	        				directives.add(d);
	        			}
	        			
	        			// Handle multiple outcomes
	        			if( directives.size() > 1  ) { 
	        				d = new RandomDirective( directives, portfolio.logger );
	        			}
	        			// handle chance item
	        			if( chance < 100 ) {
	        				d = new ChanceDirective(chance,d,portfolio.logger);
	        			} 
	        			LinkedList lld = new LinkedList();
	        			lld.add(d);
	        			Blueprint rbp;
	        			String bpName = "_"+name+"_rule"+ruleNo;
	        			rbp = new Blueprint(bpName,name+" rule "+ruleNo,false,lld,portfolio.logger);
	        			portfolio.allBlueprints.put(bpName,rbp);
	        			//portfolio.logger.info("Added rule "+bpName);
	        			writer.write("# Rule "+ruleNo+"\n");
	        			writer.write(rbp.asText()+"\n\n");
	        			continue;
	        		}
		        	portfolio.logger.warn("Unrecognised rule directive in Ruins Template: "+line);
	        		continue;
	        	}
	        	// layer (followed by layer data) endlayer
	        	if( line.matches("layer") ) {
	        		LinkedList lld = new LinkedList();
	        		// now we read in the next number of rows
	        		for( i = 0; i < length; i++ ) {
	        			String row = br.readLine();
	        			if( row == null ) { break; }
	        			if( row.matches("endlayer") ) {
	        				portfolio.logger.warn("Layer has too few rows (wanted "+length+")");
	        				break;
	        			}
	        			String items[] = row.split(",");
	        			if( items.length < width ) {
	        				portfolio.logger.warn("Layer line appears too short (wanted "+width+"): "+row);
	        			}
	        			// normal
	        			for( j = 0; (j < width)&&(j<items.length); j++ ) {
        					int rule = 0;
        					try { rule = Integer.parseInt(items[j]); } catch( Exception e ) {}
        					if( ruleOption.get(rule) != 0 ) { continue; } 
	        				if( rule == 0 ) {
		        				lld.add(new BlockDirective(Blocks.air,0,false,new BlockPos(i,0,width-j),null));
	        				} else {
	        					String callName = "_"+name+"_rule"+rule;
	        					lld.add(new CallDirective(portfolio.allBlueprints.get(callName),new BlockPos(i,0,width-j),0));
	        					//portfolio.logger.info("Adding link to "+callName);
	        				}
	        			}
	        			// delayed
	        			for( j = 0; (j < width)&&(j<items.length); j++ ) {
        					int rule = 0;
        					try { rule = Integer.parseInt(items[j]); } catch( Exception e ) {}
        					if( ruleOption.get(rule) == 0 ) { continue; } 
        					if( ruleOption.get(rule) > 2 ) { continue; } 
        					if( ruleOption.get(rule) < -2 ) { continue; } 
        					String callName = "_"+name+"_rule"+rule;
        					lld.add(new CallDirective(portfolio.allBlueprints.get(callName),new BlockPos(i,0,width-j),0));
        					//portfolio.logger.info("Adding link to "+callName);
	        			}
	        			// last
	        			for( j = 0; (j < width)&&(j<items.length); j++ ) {
        					int rule = 0;
        					try { rule = Integer.parseInt(items[j]); } catch( Exception e ) {}
        					if( (ruleOption.get(rule) > -3)&&(ruleOption.get(rule) < 3) ) { continue; } 
        					String callName = "_"+name+"_rule"+rule;
        					lld.add(new CallDirective(portfolio.allBlueprints.get(callName),new BlockPos(i,0,width-j),0));
        					//portfolio.logger.info("Adding link to "+callName);
	        			}
	        		}
	        		while( line != null && ! line.startsWith("endlayer") ) {
	        			line = br.readLine();
	        		}
        			Blueprint lbp;
        			lbp = new Blueprint("_"+name+"_layer"+layer,name+" layer "+layer,false,lld,portfolio.logger);
        			portfolio.allBlueprints.put( "_"+name+"_layer"+layer,lbp);
        			writer.write("# Layer "+layer+"\n");
        			writer.write(lbp.asText()+"\n\n");	        		
	        		layer = layer + 1;
	        		continue;
	        	}
	        	// variablename=value
	        	m = pSet.matcher(line);
	        	if( m.find() ) {
	        		if( line.startsWith("embed_into_distance") ) {
	        			try {
	        				embed = Integer.parseInt(m.group(2));
	        			} catch( Exception e) { 
	        				portfolio.logger.warn("Invalid embed_into_distance: "+m.group(2)); 
	        			}
	        			continue;
	        		}
	        		// weight (affects AUTO directive)
	        		if( line.startsWith("weight") ) {
	        			try {
	        				auto_chance = Integer.parseInt(m.group(2));
	        			} catch( Exception e) { 
	        				portfolio.logger.warn("Invalid weight: "+m.group(2)); 
	        			}
	        			continue;
	        		}
	        		// dimensions (affects layer parsing)
	        		if( line.startsWith("dimensions") ) 
	        		{
	        			String dim[] = m.group(2).split(",");
	        			try {
	        				height = Integer.parseInt(dim[0]);
	        				length = Integer.parseInt(dim[1]);
	        				width = Integer.parseInt(dim[2]);
	        			} catch( Exception e) {
	        				portfolio.logger.warn("Invalid dimensions: "+m.group(2)); 	        				
	        			}
	        			continue;
	        		}
	        		// preserve_water/lava/plants (affects soft blocks) XXXX
	        		// max_levelling and embed_into_distance (adds flattening plan) XXXX
	        		// biomesToSpawnIn (affects AUTO directive)
	        		if( line.startsWith("biomesToSpawnIn") ) {
	        			biomes = m.group(2);
	        			continue;
	        		}
	        		if( line.startsWith("max_leveling") ) { // also: max_cut_in, allowable_overhang
	        			try {
	        				maxlevel = Integer.parseInt(m.group(2));
	        			} catch( Exception e) { 
	        				portfolio.logger.warn("Invalid max_leveling: "+m.group(2)); 
	        			}
	        			continue;
	        		}
	        		continue;
	        	}
	        	portfolio.logger.warn("Unrecognised directive in Ruins Template: "+line);
	        }
	    } catch (IOException e) {
	    	portfolio.logger.error("["+fileEntry.getName()+"]: Unable to read file: "+e.getMessage());
			try {writer.close();} catch(Exception ex2) {}
	    	return false;
	    } finally {
	    	try {
	    		br.close();
	    	} catch (IOException e) {
	    		// ignore
	    	}
	    }

		// add the AUTO call
		if(auto_chance > 0 && portfolio.config.get("ruins", "withAuto", true).getBoolean(true) ) {
			steps.add( new AutoDirective(auto_chance,biomes) );
		}
		// All templates are floaters: y coord determined by embed_into_distance and ground level
		steps.add( new OptionDirective( "OPTION float "+width+","+length, portfolio.logger) );
		// add the flattening call
		if( maxlevel > 0 && portfolio.config.get("ruins", "withFlatten", true).getBoolean(true) ) {
			steps.add( new OptionDirective( "OPTION flatten 0,0 "+width+","+length+ " "+maxlevel, portfolio.logger) );
		}
		// Call the layers
		for( i = 0; i < height; i++ ) {
			Blueprint cbp = portfolio.allBlueprints.get("_"+name+"_layer"+i);
			if(cbp != null ) {
				steps.add( new CallDirective(cbp ,new BlockPos(0,i-embed,0),0 ));
			} else {
				portfolio.logger.warn("Unable to locate plan "+"_"+name+"_layer"+i);
			}
		}
		// Create the new blueprint
		bp = new Blueprint(name,description,true,steps,portfolio.logger);
		
		portfolio.allBlueprints.put( name,bp );
		if( ! portfolio.quiet ) {
			portfolio.logger.info("["+fileEntry.getName()+":"+lineNo+"]: Created new blueprint name ["+name+"] from Ruins template size (x,y,z)=("+width+","+length+","+height+")"
				+ ( (bp.auto_chance>0)?(", chance " + Math.floor(bp.auto_chance*org.steveshipway.blueprints.lib.Constants.CHANCE_BASE)):"")
		);
		}
		if( bp.auto_chance > 0 ) {
			portfolio.autoBlueprints.put( name, bp);
		}
		
		// close conversion file
		if( writer != null ) {
		    try { 
		    	writer.write("\n# Main plan\n");
		    	writer.write(bp.asText()+"\n");
		    	writer.write("# End of blueprint\n");
			} catch (IOException ex) {
				portfolio.logger.error("Unable to write to file "+cvtFileName+": "+ex.getMessage());
			}
			try {writer.close();} catch (Exception ex) {}
		}
		
		//portfolio.logger.error("TML file importing not yet supported");
		return true;
	}
	
	// Parse the inventory string:  item[#qty[#metadata[#slot]]][+item...]
	private static String inventoryToNBT(Block blk, String s, Logger logger) {
		String entityName = "Chest"; // default for chest and trapped_chest
		String nbt = "{}";
		int count;
		int slot = 0;
		int dv;
		Pattern pItem = Pattern.compile("^([^#-]+)(?:#(\\d+)(?:#(\\d+)(?:#(\\d+))?)?)?");
		if( blk == Blocks.furnace || blk == Blocks.lit_furnace ) {
			entityName = "Furnace";
		} else if( blk == Blocks.brewing_stand ) {
			entityName = "Cauldron";
		} else if( blk == Blocks.hopper ) {
			entityName = "Hopper";
		} else if( blk == Blocks.dispenser ) {
			entityName = "Trap";
		} else if( blk == Blocks.dropper ) {
			entityName = "Dropper";
		}
		nbt = "{id=\""+entityName+"\",Items=[";
		for( String item : s.split("\\+") ) {
			Matcher m = pItem.matcher(item);
			if( m.find() ) {
				String id = m.group(1);
				count = 1; slot = slot+1; dv = 0;
				try { count = Integer.parseInt(m.group(2)); } catch( Exception e ) { count = 1; }
				try { dv    = Integer.parseInt(m.group(3)); } catch( Exception e ) { dv = 0;    }
				try { slot  = Integer.parseInt(m.group(4)); } catch( Exception e ) {            }
				if( ! id.contains(":") ) { id = "minecraft:"+id; }
				nbt = nbt + "{Count="+count+"b,Slot="+slot+"b,Damage="+dv+"s,id=\""+id+"\",},";
			} else {
				logger.error("Unrecognised Inventory item in Ruins template: '"+item+"'");
			}
		}
		nbt = nbt + "],}";
		return nbt;
	}
}
