package org.steveshipway.blueprints.directive;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;

import org.apache.logging.log4j.Logger;

import net.minecraft.world.World;
import net.minecraft.init.Blocks;
import net.minecraft.block.Block;

import org.steveshipway.blueprints.Portfolio;
//Minecraft 1.7
import org.steveshipway.blueprints.util17.*;
//Minecraft 1.8
//import org.steveshipway.blueprints.util18.*;
//import net.minecraft.block.state.IBlockState;
//import net.minecraft.util.BlockPos;

public class IfBlockDirective extends Directive {
	private Set blks = null;
	private Directive directivea = null;
	private Directive directiveb = null;
	private BlockPos offset = null;
	
	public IfBlockDirective(BlockPos offset, Set blks, Directive da, Directive db, Logger logger) {
		this.directivea = da;
		this.directiveb = db;
		this.blks = blks;
		this.offset = offset;
		if( da == null ) {
    		logger.error("No 'if true' directive given to IF directive");
			return;
		}
		if( db == null ) {
    		logger.error("No 'if false' directive given to IF directive");
			return;
		}
		if(offset == null) {
			this.offset = new BlockPos(0,0,0);
		}
	}
	
	@Override
	public boolean build(World world,BlockPos location, int r, Logger logger, HashSet softlist) {
		IBlockState cbs;
		if( this.directivea == null || this.directiveb == null ) {
    		logger.error("Missing directive in IF directive");
			return false;
		}
		cbs = Util.getBlockState(world,rotateAndAdd(this.offset,r,location));
		if(this.blks.contains(cbs.getBlock())) {
			if( ! Portfolio.quiet ) { logger.info("IfBlock: Match for block type "); }
			return this.directivea.build(world,location,r,logger,softlist);
		} else {
			return this.directiveb.build(world,location,r,logger,softlist);
		}
	}
	
	@Override
	public String asText() {
		String s = "IFBLOCK ";
		int thisDv;
		Block bs = null;
		
		s += this.offset.getX()+", "+this.offset.getY()+", "+this.offset.getZ()+" ";
		
		for( Object o: this.blks ) {
			bs = (Block) o;
			s += Block.blockRegistry.getNameForObject(bs)+" ";
		}
		s += "\n  # if true\n  ";
		
		if( directivea == null) {
			s = s + "NOOP\n  # if false\n  ";
		} else {
			s = s +directivea.asText()+"\n  # if false\n  ";
		}
		if( directiveb == null) {
			s = s + "NOOP";
		} else {
			s = s +directiveb.asText();
		}
		return s;
	}
}
