package org.steveshipway.blueprints.directive;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagDouble;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.server.MinecraftServer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityList;

import net.minecraft.entity.player.EntityPlayer;

// Minecraft 1.7
import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
//import org.steveshipway.blueprints.util18.*;
//import net.minecraft.block.state.IBlockState;
//import net.minecraft.util.BlockPos;

public class PictureDirective extends Directive {
	// this is the main one!
	private BlockPos offset;
	private int dv = 0;        
	private NBTTagCompound image;
	private short mapNumber = -1;
	private String filename = null;
	private boolean hasError = false;
	private String worldName = null;
	
	public PictureDirective(int dv,BlockPos offset, String filename, NBTTagCompound image) {
		this.offset = offset;
		this.image = image;
		this.dv = dv;
		this.filename = filename;
	}
	public PictureDirective(String definition, String path, Logger logger) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*(?:picture|pic|img|image))?\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s+(\\d)\\s+([^\\s]+\\.(jpg|png|gif))");
    	Matcher m = p.matcher(definition);
    	if( m.find() ) {
    		int dv = 0;
			// coordinates
    		try {
    			this.offset = new  BlockPos(Integer.parseInt( m.group(1) ),Integer.parseInt( m.group(2) ),Integer.parseInt( m.group(3) ));
    		} catch( Exception e ) {
    			this.hasError = true;
        		logger.error("Cannot parse PICTURE definition: "+definition);
        		return;
    		}

			if( m.group(4) != null ) {
    			dv = Integer.parseInt( m.group(4) );
    		} else { dv = 0; }
    		this.dv = dv;

    		this.filename = path + File.separator + "img" + File.separator + m.group(5);
    		
    		File f = new File(this.filename);
    		if( ! f.exists() ) {
    			this.hasError = true;
    			logger.error("File "+this.filename + " does not exist in PICTURE definition.");
    			return;
    		}
    		
    		// should we parse the file into the NBT here, or when required?
    		//this.mapNumber = MapImage.createMap(MinecraftServer.getServer().worldServers[0],this.filename);
    		// Only when required!  Otherwise we create a new map ever time we load...!
		
    	} else {
    		this.hasError = true;
    		logger.error("Cannot parse PICTURE definition: "+definition);
    	}
	}
	
	@Override
	public boolean build(World world, BlockPos location, int r, Logger logger, HashSet softlist) {
		boolean rv;
		BlockPos blockPos = new BlockPos(location);
		NBTTagCompound nbtTag;
		NBTTagCompound nbtMap;
		Entity entity;
		int tileX, tileY, tileZ;
		byte facing;
		
		if( world.isRemote ) {
    		logger.error("Block placement called with remote world?  Should never happen.");
			return false;
		}
		if( this.hasError ) {
			logger.error("Picture definition has an error, cannot spawn");
			return false;
		}

		// get block position, rotated
		blockPos = rotateAndAdd(this.offset,r,location);
		
		// A Picture is a frame surrounding a filled_map.

		// First, create the filled_map object, if not done before in this world this process
		if((this.mapNumber < 0) || (this.worldName == null) || (!this.worldName.contentEquals(world.getWorldInfo().getWorldName()) ) ) {
			this.mapNumber = MapImage.createMap(world,this.filename,logger);
			if(this.mapNumber < 0 ) {
				logger.error("Unable to convert image file into Map format");
				return false;
			}
			this.worldName = world.getWorldInfo().getWorldName();
		} // else, we already have this.
		
		//ItemStack map = new ItemStack(Items.filled_map,1,this.mapNumber); // are params in correct order (qty, DV)
		nbtMap = new NBTTagCompound();
		
		nbtMap.setByte("Count",(byte) 1);
		nbtMap.setShort("Damage",this.mapNumber);
		
		// Now, create the frame entity.
		nbtTag = new NBTTagCompound();
		nbtTag.setString("id","ItemFrame");
		// set map to be frame content
		nbtTag.setTag("Item",nbtMap);
		// set frame options
		nbtTag.setByte("ItemRotation",(byte) 0);
		nbtTag.setFloat("ItemDropChance",1.0F);
		// rotation in world
		facing = (byte)((this.dv + r)&3);
		
		// Minecraft 1.7
		nbtMap.setShort("id",(short)358);
		// Minecraft 1.8
		//nbtMap.setString("id","filled_map");
		
		// Minecraft 1.8
		//nbtTag.setByte("Facing",facing);
		//tileX = blockPos.getX(); // the block it is IN
		//tileY = blockPos.getY();
		//tileZ = blockPos.getZ();
		// Minecraft 1.7  // the block it is ON
		nbtTag.setByte("Direction",facing);
		tileY = blockPos.getY();
		switch( facing ) {
			case 3: // e
				tileX = blockPos.getX()-1;
				tileZ = blockPos.getZ();				
				break;
			case 2: // n ( towards us )
				tileX = blockPos.getX();
				tileZ = blockPos.getZ()+1;
				break;
			case 1: // w
				tileX = blockPos.getX()+1;
				tileZ = blockPos.getZ();
				break;
			default:  // s
				tileX = blockPos.getX();
				tileZ = blockPos.getZ()-1;
				break;
		}

		nbtTag.setInteger("TileX", tileX);
		nbtTag.setInteger("TileY", tileY);
		nbtTag.setInteger("TileZ", tileZ);
		
		// set the position tags
		NBTTagList posList = new NBTTagList();
		posList.appendTag(new NBTTagDouble((double)blockPos.getX()+0.5D));
		posList.appendTag(new NBTTagDouble((double)blockPos.getY()+0.5D));
		posList.appendTag(new NBTTagDouble((double)blockPos.getZ()+0.5D));
		nbtTag.setTag("Pos",posList);
		
		// create the entity
		entity = EntityList.createEntityFromNBT(nbtTag, world);
		if( entity == null ) {
			logger.error("Unable to create Frame Entity");
		} else { 
			entity.setPosition((double) blockPos.getX()+0.5D,(double) blockPos.getY(),(double) blockPos.getZ()+0.5D);		
			world.spawnEntityInWorld(entity);
		}
		
		return true;
	}
	
	@Override
	public String asText() {
		String s = "PICTURE ";
		if( this.hasError ) {
			s = "# PICTURE ";
		}
		s += " " + offset.getX() + "," + offset.getY() + "," + offset.getZ() + "  " + this.dv + " " + this.filename;
		return s;
	}
}
