package org.steveshipway.blueprints;

import java.util.Arrays;
import java.util.HashSet;

import org.steveshipway.blueprints.directive.*;
import org.steveshipway.blueprints.lib.Constants;

import net.minecraft.block.Block;
import net.minecraft.nbt.*;
import net.minecraft.init.Blocks;
import net.minecraft.world.World;

//Minecraft 1.7
//import org.steveshipway.blueprints.util17.*;
//Minecraft 1.8
import org.steveshipway.blueprints.util18.*;
import net.minecraft.util.BlockPos;
import net.minecraft.block.state.IBlockState;

// One of the steps required to perform an Undo
public class UndoStep {
	private BlockPos location;
	private IBlockState blk;
	private NBTTagCompound tag = null;
	
	static private HashSet<Block> decorations = new HashSet<Block>(
			Arrays.asList( 
					Blocks.torch,
					Blocks.rail,
		    		Blocks.redstone_torch,	    		
		    		Blocks.redstone_wire,
		    		Blocks.wall_banner,
		    		Blocks.standing_banner,
		    		Blocks.carpet,
		    		Blocks.skull,
		    		Blocks.tripwire,
		    		Blocks.lever,
		    		Blocks.stone_button,	Blocks.wooden_button,
		    		Blocks.leaves,	Blocks.leaves2,
		    		Blocks.yellow_flower, Blocks.red_flower,
		    		Blocks.double_plant,
		    		Blocks.wheat,
		    		Blocks.pumpkin_stem,
		    		Blocks.melon_stem,
		    		Blocks.waterlily,
		    		Blocks.reeds,
		    		Blocks.cactus,
		    		Blocks.potatoes,
		    		Blocks.tallgrass,
		    		Blocks.fire
			)
	);
	
	public UndoStep(BlockPos location,IBlockState blk,NBTTagCompound tag) {
		this.blk = blk;
		this.location = new BlockPos(location);
		if( tag != null ) {
			this.tag = (NBTTagCompound) tag.copy();
		}
	}
	public boolean apply(World world) {
		boolean rv;
		rv = Util.setBlockState(world,this.location,this.blk);
		if( !rv ) {
			return false;
		}		
		if( this.tag != null ) {
			Util.setTileEntity(world,this.location,this.tag);
		}
		return true;
	}
	// use type 0 for blocks, type 1 for decorations
	public boolean apply(World world, int type) {
		boolean rv = true;
		if( decorations.contains(this.blk.getBlock())) {
			if( type > 0 ) { rv = this.apply(world); }
		} else {
			if( type < 1 ) { rv = this.apply(world); }
		}
		return rv;
	}
}
