package org.steveshipway.blueprints.directive;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;




import org.steveshipway.blueprints.Portfolio;
// Minecraft 1.7
//import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
import org.steveshipway.blueprints.util18.*;

import net.minecraft.util.BlockPos;
import net.minecraft.block.state.IBlockState;

public class AreaDirective extends Directive {
	protected BlockPos locationa;
	protected BlockPos locationb;
	protected boolean soft = false;
	protected IBlockState block = null;
	protected String nbt = null; // possibly this should be a parsed NBT object
	protected int dv = 0;
	protected NBTTagCompound tileNbt;
	protected String meta = null;
	
	public AreaDirective(String definition, String bppath, Logger logger) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*area)?\\s*(~?)((?:\\w+:)?\\w+)(?:\\((\\d+)\\))?\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)(?:\\s+(\\{.*\\}|[^\\s]+\\.nbt))?");
    	Matcher m = p.matcher(definition);
    	if( m.find() ) {
    		int dv = 0;
    		Block b = null;
    	
    		if( m.group(1).contains("~") ) {
    			this.soft = true;
    		}
    		
    		if( m.group(3) != null ) {
    			dv = Integer.parseInt( m.group(3) );
    		} else { dv = 0; }
    		if( m.group(2).matches("^((OLD_)?STONE(_(STEPS|STAIRS|SLAB))?|WOOD(_(STEPS|STAIRS|SLAB))?|PLANKS|DOOR|GLASS(_PANE)?|FENCE|GATE)$") ) {
        		this.block = null;
        		this.meta = m.group(2);
    		} else {
    			if( m.group(2).contains(":") ) {
    				b = Block.getBlockFromName(m.group(2));
    			} else {
    				b = Block.getBlockFromName("minecraft:"+m.group(2));    				
    			}
	    		if( b == null ) {
	    			// invalid name
	        		logger.error("Invalid block name in AREA def: "+m.group(2));
	        		b = Block.getBlockFromName("minecraft:air");
	    		} 
	    		this.block = Util.getState(b, dv);
    		}
    		this.dv = dv; // not really needed, but...
    			
    		// 2 is the block, 3 is the optional DV
	    	this.locationa = new BlockPos(Integer.parseInt( m.group(4) ),Integer.parseInt( m.group(5) ),Integer.parseInt( m.group(6) ));	
	    	this.locationb = new BlockPos(Integer.parseInt( m.group(7) ),Integer.parseInt( m.group(8) ),Integer.parseInt( m.group(9) ));
    		
			this.nbt = m.group(10);
			if( this.nbt != null ) {
				this.tileNbt = Util.parseNBT(this.nbt,bppath,logger);
			} else { this.tileNbt = null; }
    	} else {
    		logger.error("Cannot parse AREA definition: "+definition);
    	}
	}

	@Override
	public boolean build(World world,BlockPos location, int r, Logger logger, HashSet softlist) {
		BlockPos blockPosA = null;
		BlockPos blockPosB = null;
		int stepx, stepy, stepz;
		boolean rv = true;
		if( block == null && this.meta == null ) {
    		logger.error("No valid block definition in AREA directive");
			return false;
		}
		if( world.isRemote ) {
    		logger.error("Block placement called with remote world?  Should not happen.");
			return false;
		}
		
		//if( this.soft ) {
		//	logger.info("Softlist: ");
		//	for( Object o : softlist ) {
		//		logger.info(Block.blockRegistry.getNameForObject((Block)o));
		//	}
		//}
		
		// rotate offset by r
		blockPosA = rotateAndAdd(this.locationa,r,location);
		blockPosB = rotateAndAdd(this.locationb,r,location);
		if( blockPosA.getX() > blockPosB.getX() ) { stepx = -1; } else { stepx = 1; }
		if( blockPosA.getY() > blockPosB.getY() ) {stepy = -1; } else { stepy = 1; }
		if( blockPosA.getZ() > blockPosB.getZ() ) {stepz = -1; } else { stepz = 1; }

		// place the block(s), possibly with the NBT in a tileentity
		// iterate between the two locations
		BlockPos bp;
		Chunk chk;
		Block curBlock = null;
		IBlockState blockstate = null;
		IBlockState newstate = null;
		int curDv = 0;
		int ir = r;
		NBTTagCompound nbtTag = null;
		
		if( this.tileNbt != null ) {
			nbtTag = (NBTTagCompound) this.tileNbt.copy();
		}
		if( this.block != null ) {
			newstate = this.block;
			newstate = Util.rotateState(newstate,r);
		}
		for( int x = blockPosA.getX(); x != (blockPosB.getX()+stepx); x += stepx ) {
			for( int y = blockPosA.getY(); y != (blockPosB.getY()+stepy); y += stepy ) {
				for( int z = blockPosA.getZ(); z != (blockPosB.getZ()+stepz); z += stepz ) {
					bp = new BlockPos(x,y,z);
					blockstate = Util.getBlockState(world,bp);
					curBlock = blockstate.getBlock(); 
					curDv = Util.getDvFromState(blockstate);
					if(this.soft) {
						if( ! softlist.contains(curBlock) ) {
//						if( curBlock != Blocks.air && curBlock != Blocks.water && curBlock != Blocks.flowing_water ) {
							continue;  // soft block only placed in air or water
						}
					}
					
					if( this.meta != null ) { // must recalculate the newstate each time as this is a metablock
						newstate = Util.metaBlock(world,bp,this.meta,this.dv);						
						newstate = Util.rotateState(newstate,r);
					}
	
					Portfolio.undo.add(world,bp);
					
					if( curBlock != newstate.getBlock() || curDv != Util.getDvFromState(newstate) ) { // newstate.getBlock().getMetaFromState(newstate) ) {
						rv = Util.setBlockState(world,bp,newstate);
					  	if( !rv ) {
					  		logger.warn("Failed to place block in the world!");
					  		return false;
					  	}
					  	// if there was NBT specified, create tile entity
						if( nbtTag != null ) {
							nbtTag.setInteger("x", x);
							nbtTag.setInteger("y", y);
							nbtTag.setInteger("z", z);
							//TileEntity te = TileEntity.createAndLoadEntity(nbtTag);
							Util.setTileEntity(world,bp,nbtTag);
						}
					}
				}
				
			}
		}
		return true;
	}
	
	@Override
	public String asText() {
		String s = "AREA ";
		if( block == null) {
			return "# AREA undefined ";
		}
		if(soft) { 
			s += "~"; 
		}
		s += Block.blockRegistry.getNameForObject(block);
		s += " " + locationa.getX() + "," + locationa.getY() + "," + locationa.getZ() ;
		s += " " + locationb.getX() + "," + locationb.getY() + "," + locationb.getZ() ;
		if( nbt != null ) {
			s += " " + nbt;
		}
		return s;
	}
	
	
}
