package org.steveshipway.blueprints.directive;

import java.util.HashSet;
import java.util.Random;

import org.apache.logging.log4j.Logger;

import net.minecraft.world.World;


//Minecraft 1.7
//import org.steveshipway.blueprints.util17.*;
//Minecraft 1.8
import org.steveshipway.blueprints.util18.*;

import net.minecraft.util.BlockPos;

public class IfBiomeDirective extends Directive {
	private String match = null;
	private Directive directivea = null;
	private Directive directiveb = null;
	
	public IfBiomeDirective(String match, Directive da, Directive db, Logger logger) {
		this.directivea = da;
		this.directiveb = db;
		this.match = match;
		if( da == null ) {
    		logger.error("No 'if true' directive given to IFBIOME directive");
			return;
		}
		if( db == null ) {
    		logger.error("No 'if false' directive given to IFBIOME directive");
			return;
		}
		if( match == null ) {
    		logger.error("No match string given to IFBIOME directive");
			return;
		}
	}
	
	@Override
	public boolean build(World world,BlockPos location, int r, Logger logger, HashSet softlist) {
		String biome;
		if( this.directivea == null || this.directiveb == null ) {
    		logger.error("Missing directive in IF directive");
			return false;
		}
		biome = Util.getBiome(world,location);
		if(biome.matches(".*("+this.match+").*")) {
			return this.directivea.build(world,location,r,logger,softlist);
		} else {
			return this.directiveb.build(world,location,r,logger,softlist);
		}
	}
	
	@Override
	public String asText() {
		String s = "IFBIOME "+ this.match + "\n  ";
		
		if( directivea == null) {
			s = s + "NOOP\n  ";
		} else {
			s = s +directivea.asText()+"\n  ";
		}
		if( directiveb == null) {
			s = s + "NOOP";
		} else {
			s = s +directiveb.asText();
		}
		return s;
	}
}
