package org.steveshipway.blueprints.directive;

import java.util.HashSet;

import net.minecraft.util.BlockPos;
import net.minecraft.world.World;

import org.apache.logging.log4j.Logger;
import org.steveshipway.blueprints.util18.Util;

public class IfYDirective extends Directive {
	private int value = 0;
	private char cmp = '=';
	private Directive directivea = null;
	private Directive directiveb = null;
	
	public IfYDirective(char cmp, int val, Directive da, Directive db, Logger logger) {
		this.directivea = da;
		this.directiveb = db;
		this.value = val;
		this.cmp = cmp;
		if( da == null ) {
    		logger.error("No 'if true' directive given to IFY directive");
			return;
		}
		if( db == null ) {
    		logger.error("No 'if false' directive given to IFY directive");
			return;
		}
		if( cmp != '<' && cmp != '>' && cmp != '=' ) {
    		logger.error("Incorrect comparator given to IFY directive");
			return;
		}
	}
	
	@Override
	public boolean build(World world,BlockPos location, int r, Logger logger, HashSet softlist) {
		boolean rv = true;
		if( this.directivea == null || this.directiveb == null ) {
    		logger.error("Missing directive in IFY directive");
			return false;
		}
		if( this.cmp == '=' ) {
			rv = ( location.getY() == this.value );
		} else if( this.cmp == '<' ) {
			rv = ( location.getY() > this.value );
		} else if( this.cmp == '>' ) {
			rv = ( location.getY() < this.value );
		} else {
    		logger.error("Invalid comparator ("+this.cmp+") in IFY directive");
			return false;
		}
		if(rv) {
			return this.directivea.build(world,location,r,logger,softlist);
		} else {
			return this.directiveb.build(world,location,r,logger,softlist);
		}
	}
	
	@Override
	public String asText() {
		String s = "IFY "+ this.cmp + " " + this.value + "\n  ";
		
		if( directivea == null) {
			s = s + "NOOP\n  ";
		} else {
			s = s +directivea.asText()+"\n  ";
		}
		if( directiveb == null) {
			s = s + "NOOP";
		} else {
			s = s +directiveb.asText();
		}
		return s;
	}
}
