package org.steveshipway.blueprints.directive;

import java.util.HashSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagDouble;
import net.minecraft.nbt.NBTTagFloat;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityList;


// Minecraft 1.7
//import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
import org.steveshipway.blueprints.util18.*;

import net.minecraft.block.state.IBlockState;
import net.minecraft.util.BlockPos;

public class MobDirective  extends Directive {
	private String mobdata;
	private NBTTagCompound nbt = null;
	private BlockPos offset;
	private boolean haserror = false;
	
	static Pattern nbtpattern = Pattern.compile("(?i)^\\s*(?:mob|entity)?\\s*(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s*(\\S+\\.nbt|\\{.*\\})\\s*$");
	
	public MobDirective(String definition, String bppath,Logger logger) {
		// parse options
		// MOB x y z filename.nbt
		// MOB x y z { nbt }
		// MOB x y z name
		Matcher m;
		
		this.mobdata = definition;
		
		// parse explicit NBT for the spawner
		m = nbtpattern.matcher(definition);
		if ( m.find() ) {
			this.mobdata = m.group(4);
			try {
				this.offset = new BlockPos(Integer.parseInt(m.group(1)),Integer.parseInt(m.group(2)),Integer.parseInt(m.group(3)));
			} catch ( Exception e ) {
				logger.error("Invalid offset format in MOB definition '"+definition+"'");
				this.haserror = true;
				return;
			}
			this.nbt = Util.parseNBT(m.group(4),bppath,logger);
			if( this.nbt == null ) {
				logger.error("Supplied NBT in "+m.group(4)+" does not parse");
				this.haserror = true;			
				return;
			}
			if( this.nbt.getString("id") == null || this.nbt.getString("id").length()<1 ) {
				logger.error("Supplied NBT in "+m.group(4)+" does not specify an id");
				this.haserror = true;
			}
			return;
		} else {
			logger.error("Invalid format in MOB definition '"+definition+"'");
			this.haserror = true;			
		}
	}
	public MobDirective(BlockPos offset, NBTTagCompound nbt) {
		this.offset = offset;
		this.nbt = nbt;
		this.mobdata = nbt.toString();
		//CheshireCat.logger.info("Creating new mob directive: "+offset.getX()+","+offset.getY()+","+offset.getZ()+" : "+nbt.toString());
	}
	
	@Override
	public boolean build(World world, BlockPos location, int r, Logger logger, HashSet softlist) {
		IBlockState blockState = null;
		BlockPos blockPos = new BlockPos(location); 
		boolean rv;
		NBTTagCompound nbtTag = null;
		Entity entity;
		
		if( world.isRemote ) {
    		logger.error("Mob placement called with remote world?  Should never happen.");
			return false;
		}
		if( this.haserror || (this.nbt==null) ) {
			logger.error("Mob definition has an error, cannot spawn");
			return false;
		}
		
		// get block position, rotated
		blockPos = rotateAndAdd(this.offset,r,location);

		// set the position tags
		nbtTag = (NBTTagCompound) this.nbt.copy();
		//logger.info("Searching for mob direction setting on "+nbtTag.getString("id"));
		if( nbtTag.hasKey("Direction") ) {
			// There should be a Float list called Direction,
			// where the 1st item is rotation in degrees.
			//logger.info("Direction key found");
			float angle = 0.0f;
			NBTTagList direction = nbtTag.getTagList("Direction",5);
			if( direction != null ) {
				//logger.info("Diraction Taglist found");
				angle = direction.getFloat(0); 
				angle = angle + (90.0f * r);
				if( angle > 360.0f ) { angle -= 360.0f; }
				direction.set(0,(NBTBase) new NBTTagFloat(angle));
				nbtTag.setTag("Direction",direction);
			} else {
				// legacy support
				//logger.info("Legacy support enabled");
				byte dir = nbtTag.getByte("Direction");
				dir = (byte)((dir + r)&3);
				nbtTag.setByte("Direction",  dir );
				nbtTag.setByte("Dir",  dir );
			}
		}
		if( nbtTag.hasKey("Rotation") ) {
			// There should be a Float list called Rotation,
			// where the 1st item is rotation in degrees.
			logger.info("Rotation key found");
			float angle = 0.0f;
			NBTTagList rotation = nbtTag.getTagList("Rotation",5);
			if( rotation != null ) {
				angle = rotation.getFloat(0); 
				logger.info("Rotation Taglist found: "+angle);
				angle = angle + (90.0f * r);
				if( angle > 360.0f ) { angle -= 360.0f; }
				logger.info("Change to: "+angle);
				rotation.set(0,(NBTBase) new NBTTagFloat(angle));
				nbtTag.setTag("Rotation",rotation);
			}
		}
		if( nbtTag.hasKey("Facing") ) {
			//logger.info("Facing key found");
			byte dir = nbtTag.getByte("Facing");
			dir = (byte)((dir + r)&3);
			nbtTag.setByte("Facing",  dir );
		}
		NBTTagList posList = new NBTTagList();
		posList.appendTag(new NBTTagDouble((double)blockPos.getX()+0.5D));
		posList.appendTag(new NBTTagDouble((double)blockPos.getY()));
		posList.appendTag(new NBTTagDouble((double)blockPos.getZ()+0.5D));
		nbtTag.setTag("Pos",posList);
		// Minecraft 1.8 : TileX/Y/Z are the block we are IN
		if( nbtTag.getString("id").matches("ItemFrame|Painting")) {
			nbtTag.setInteger("TileX",blockPos.getX());
			nbtTag.setInteger("TileY",blockPos.getY());
			nbtTag.setInteger("TileZ",blockPos.getZ());
		}
		// Minecraft 1.7 : TileX/Y/Z are the block we are ON
		//if( nbtTag.getString("id").matches("ItemFrame|Painting")) {
		//	byte dir = nbtTag.getByte("Direction");
		//	nbtTag.setInteger("TileX",blockPos.getX()+((dir==1)?1:0)+((dir==3)?-1:0));
		//	nbtTag.setInteger("TileY",blockPos.getY());
		//	nbtTag.setInteger("TileZ",blockPos.getZ()+((dir==2)?1:0)+((dir==0)?-1:0));
		//}
		
		// create the entity
		entity = EntityList.createEntityFromNBT(nbtTag, world);
		if( entity == null ) {
			logger.error("Unable to create Entity from the NBT: '"+this.nbt.toString()+"'");
			//return false;
		} else {
			entity.setPosition((double) blockPos.getX()+0.5D,(double) blockPos.getY(),(double) blockPos.getZ()+0.5D);		
			world.spawnEntityInWorld(entity);
			//logger.info("Spawning in the world: "+this.nbt.getString("id"));
		}
		
		return true;
	}
	
	@Override
	public String asText() {
		String s = "MOB ";
		if( this.haserror || (this.nbt == null)) {
			return "# MOB undefined ";
		}
		s += " " + offset.getX() + "," + offset.getY() + "," + offset.getZ();
		if( this.mobdata != null ) {
			s += " " + this.mobdata;
		}
		return s;
	}
}
