package org.steveshipway.cheshirecat.item;

import java.util.List;

import net.minecraft.item.Item;

import org.steveshipway.blueprints.Blueprint;
import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.lib.Constants;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.entity.RenderItem;
import net.minecraft.client.resources.model.ModelResourceLocation;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.MathHelper;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;

// Minecraft 1.7
//import cpw.mods.fml.common.registry.GameRegistry;
//import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraft.util.BlockPos;
import org.steveshipway.blueprints.util18.*;

public class ItemWand extends Item {
	public static String name = "wand";
	boolean enabled = true;
	boolean copyMode = false;
	String copyName = null;
	BlockPos copyPos = null;
	
    public ItemWand(){
    	this.setUnlocalizedName(Constants.MODID + "_" + name);
    	// now done with insanely complex resource definitions
    	this.setCreativeTab(CreativeTabs.tabTools);
    	this.setMaxStackSize(1);
    	this.setHarvestLevel("wand",3);
    	//this.setTextureName(Constants.MODID + ":" + name);
    }
    
    public void registerItem() {
    	GameRegistry.registerItem(this, this.name);    	
    }
    public void registerRender() {
    	Minecraft.getMinecraft().getRenderItem().getItemModelMesher().register(
    			this, 
    			0, 
    			new ModelResourceLocation(Constants.MODID + ":" + this.name, "inventory")
    	);
    }
    
    public String getName() {
    	return name;
    }
    public void enable() { 
    	enabled = true;
    }
    public void disable() { 
    	enabled = false;
    }
    public boolean onItemUse(ItemStack item, EntityPlayer player, World world,
    	// which block was in the target when clicked
// Minecraft 1.7
//        int posx, int posy, int posz,
//        int sideint,
// Minecraft 1.8
    	BlockPos pos, 
    	EnumFacing side, 
    	// where on the target block was clicked (0.0-1.0)
    	float blockx, float blocky, float blockz)
    {
        // Minecraft 1.7
        //BlockPos pos = new BlockPos(posx,posy,posz);
        //EnumFacing side = Util.enumFromInt(sideint);

        if(world.isRemote) {
    	 	// can't do anything here as we don't own the world (client side)
            return false;
        }
    	
        BlockPos target = new BlockPos(pos);
	    int rotation = MathHelper.floor_double((double)(player.rotationYaw * 4.0F / 360.0F) + 0.5D) & 3;
	    switch( side ) {
		case DOWN:
        	target = pos.down();
			break;
		case EAST:
        	target = pos.east();
			break;
		case NORTH:
        	target = pos.north();
			break;
		case SOUTH:
        	target = pos.south();
			break;
		case WEST:
        	target = pos.west();
			break;
		case UP:
        	target = pos.up();
			break;
	    }
    	
        if( this.copyMode ) {
        	if( this.copyPos == null ) {
        		this.startCopy(player,target);
        	} else {
        		this.endCopy(player,item,target);
        	}
        	return true;
        }
        
    	NBTTagCompound nbt = item.getTagCompound();
    	if( nbt == null ) {  // no compound data?
    		player.addChatMessage(new ChatComponentText("Wand is not yet enchanted"));
    		return true;
    	}
    	String blueprint = nbt.getString("blueprint");
        if( blueprint == null || blueprint.matches("") ) {
    		// no blueprint is set...
    		player.addChatMessage(new ChatComponentText("Wand is not yet enchanted"));
    		return true;
    	}
   	    // activate the wand!
        if( ! enabled ) {
    		player.addChatMessage(new ChatComponentText("Wait for last build to finish..."));
    		return true;
        }
        enabled = false;        
		//player.addChatMessage(new ChatComponentText("Building on side "+side+" when facing direction "+rotation));
        if( CheshireCat.portfolio.build(world, blueprint, target, rotation) ) {
    		player.addChatMessage(new ChatComponentText("Build was successful"));
        } else {
    		player.addChatMessage(new ChatComponentText("Build failed!"));
        }
        enabled = true;
        return true;
    }
    
    
    public void onCreated(ItemStack itemStack, World world, EntityPlayer player) {
        NBTTagCompound nbt = new NBTTagCompound();
        nbt.setString("blueprint", "");
        nbt.setBoolean("locked",false);
        itemStack.setTagCompound(nbt);
    }
    
    @Override
    public boolean isItemTool(ItemStack itemstack)  {
    	return true;
    }
    
    @Override
    public boolean canItemEditBlocks() {
    	return true;
    }
    
    public void addInformation(ItemStack par1ItemStack, EntityPlayer par2EntityPlayer, List par3List, boolean par4) {
    	NBTTagCompound nbt = par1ItemStack.getTagCompound();
    	if( nbt == null ) {
    		nbt = new NBTTagCompound();
    		nbt.setString("blueprint", "");
            nbt.setBoolean("locked",false);
    		par1ItemStack.setTagCompound(nbt);
    		par3List.add(EnumChatFormatting.RED + "Not initialised");
    		if( this.copyMode ) {
        		par3List.add(EnumChatFormatting.RED + "COPYING");    			
    		}
    		return;
    	}
    	String blueprint = nbt.getString("blueprint");
    	if( blueprint != null && !blueprint.matches("")) {
    		Blueprint bp;
    		bp = CheshireCat.portfolio.blueprint(blueprint);
    		par3List.add(EnumChatFormatting.GREEN + "Creates: " + blueprint);
    		if( bp != null ) {
    			par3List.add(EnumChatFormatting.AQUA + CheshireCat.portfolio.blueprint(blueprint).helpText() );
    		} else {
        		par3List.add(EnumChatFormatting.RED + "INVALID BLUEPRINT" );    		
    		}
    	} else {
    		par3List.add(EnumChatFormatting.AQUA + "Not yet enchanted");    	
    		//par3List.add(EnumChatFormatting.AQUA + "Use /wand set <blueprint>");    	
    	}
		if( this.copyMode ) {
    		par3List.add(EnumChatFormatting.RED + "COPYING DATA");    			
		}
    }
    
    public int getItemEnchantability()
    {
        return 50; // it's a wand...
    }
    
    public void initCopyMode(String bpname ) {
    	this.copyMode = true;
    	this.copyName = bpname;
    	this.copyPos = null;    	
    }
    private void startCopy(EntityPlayer player, BlockPos bp) {
    	this.copyPos = bp;
    	player.addChatMessage(new ChatComponentText("First coordinate saved.  Now right-click on the other base corner to start copy."));
    }
    private void endCopy(EntityPlayer player, ItemStack item, BlockPos bp) {
    	BlockPos from = null;
    	BlockPos to = null;
    	int x1,y1,z1,x2,y2,z2;
    	int i,j;
    	int y;
    	
    	x1 = this.copyPos.getX();
    	y1 = this.copyPos.getY();
    	z1 = this.copyPos.getZ();
    	x2 = bp.getX();
    	y2 = bp.getY();
    	z2 = bp.getZ();
    	// identify max and min corners, and lowest point
    	if( y2 < y1 ) { i = y1; y1 = y2; y2 = i; } 
    	if( x2 < x1 ) { i = x1; x1 = x2; x2 = i; }
    	if( z2 < z1 ) { i = z1; z1 = z2; z2 = i; }
    	// calculate height
    	for( i = x1; i <= x2; i++ ) {
        	for( j = z1; j <= z2; j++ ) {
        		y = 128;
        		while( (y>y2) && Util.isAirBlock(player.worldObj,new BlockPos(i,y,j)) ) {
        			y--;
        		}
        		y2 = y;
        	}    		
    	}
    	player.addChatMessage(new ChatComponentText("Calculated area height as Y="+y2));
    	from = new BlockPos(x1,y1,z1);
    	to = new BlockPos(x2,y2,z2);

    	// call copy function to create blueprint
  		player.addChatMessage(new ChatComponentText("Creating new blueprint '"+this.copyName+"'"));
  	  	if(CheshireCat.portfolio.copy(player.worldObj, this.copyName, from, to, 0) != null) {
  	  		player.addChatMessage(new ChatComponentText("Copy created successfully."));
  	  		player.addChatMessage(new ChatComponentText("Now right-click to paste, or use '/bp save "+this.copyName+"' to save."));
  	    	NBTTagCompound nbt = item.getTagCompound();
  	    	if( nbt == null ) {
  	    		nbt = new NBTTagCompound();
  	            nbt.setBoolean("locked",false);
  	    	}
    		nbt.setString("blueprint", this.copyName);
    		item.setTagCompound(nbt);
  	  	} else {
  	  		player.addChatMessage(new ChatComponentText("Unable to copy block pattern"));
	  	}
    	// reset wand to normal
    	this.copyMode = false;
    	this.copyPos = null;
    	this.copyName = null;
    }
}
